@@ -32,7 +32,7 @@ import threading
 import http.cookies as cookies
 import datetime
 
-from typing import Any, Callable, Dict, List, Union
+from typing import Any, Callable, Dict, List, Tuple, Union
 
 from simple_http_server import FilterContex, ModelDict, Environment, RegGroup, RegGroups, HttpError, RequestBodyReader, StaticFile, \
     Headers, Redirect, Response, Cookies, Cookie, JSONBody, BytesBody, Header, Parameters, PathValue, \
@@ -126,6 +126,7 @@ class ResponseWrapper(Response):
         super().__init__(status_code=status_code, headers=headers, body="")
         self.__req_handler = handler
         self.__is_sent = False
+        self.__header_sent = False
         self.__send_lock__ = threading.Lock()
 
     @property
@@ -150,7 +151,8 @@ class ResponseWrapper(Response):
             self.__send_response()
 
     def __send_response(self):
-        assert not self.__is_sent, "This response has benn sent"
+        assert not self.__is_sent and not self.__header_sent, "This response has benn sent"
+        self.__header_sent = True
         self.__is_sent = True
         self.__req_handler._send_response({
             "status_code": self.status_code,
@@ -159,6 +161,22 @@ class ResponseWrapper(Response):
             "body": self.body
         })
 
+    def __send_headers(self):
+        if not self.__header_sent:
+            self.__header_sent = True
+            self.__req_handler._send_res_headers(self.status_code, headers=self.headers, cks=self.cookies)
+
+    def write_bytes(self, data: bytes):
+        assert not self.__is_sent, "This response has benn sent"
+        assert isinstance(data, bytes) or isinstance(data, bytearray), "You can "
+        self.__send_headers()
+        self.__req_handler.writer.write(data)
+
+    def close(self):
+        self.__is_sent = True
+        self.__req_handler.writer.write_eof()
+        self.__req_handler.writer.close()
+
 
 class FilterContexImpl(FilterContex):
     """Context of a filter"""
@@ -276,7 +294,7 @@ class FilterContexImpl(FilterContex):
                 headers.update(item)
             elif isinstance(item, cookies.BaseCookie):
                 cks.update(item)
-            elif type(item) in (str, dict, StaticFile, bytes):
+            elif type(item) in (str, dict, StaticFile, bytes, bytearray):
                 if body is None:
                     body = item
         return status_code, headers, cks, body
@@ -569,13 +587,81 @@ class HTTPRequestHandler:
         self.writer = http_protocol_handler.writer
         self.environment: Dict[str, Any] = environment
 
+    def __match_one_exp(self, d: Dict[str, Union[List[str], str]], exp: str, where: str) -> bool:
+        if not exp:
+            return True
+        _logger.debug(f"Match controller {where} expression [{exp}] for values: {d}. ")
+        exp_ = str(exp)
+        if exp_.find("=") < 0:
+            _logger.debug(f"cannot find = in exp {exp}")
+            if exp_.startswith('!'):
+                return not exp_[1:] in d.keys()
+            else:
+                return exp_ in d.keys()
+        idx = exp_.find("!=")
+        if idx > 0:
+            k, v = utils.break_into(exp_, '!=')
+            if k not in d.keys():
+                _logger.debug(f"{k} is not in {d}")
+                return False
+            dvals = d[k]
+            if isinstance(dvals, str):
+                dvals = [dvals]
+            for dv in dvals:
+                if dv == v:
+                    _logger.debug(f"There is a {dv} is not in {d}")
+                    return False
+            return True
+
+        idx = exp_.find('=')
+        if idx > 0:
+            k, v = utils.break_into(exp_, '=')
+            if k not in d.keys():
+                return False
+            dvals = d[k]
+            if isinstance(dvals, str):
+                dvals = [dvals]
+            for dv in dvals:
+                if dv == v:
+                    return True
+            return False
+
+        _logger.error(f"Controller {where} expression [{exp}] is not valied, returning False...")
+        return False
+
+    def __match_exps(self, d: Dict[str, Union[List[str], str]], exps: List[str], all: bool, where: str) -> bool:
+        if not exps:
+            return True
+
+        for exp in exps:
+            res = self.__match_one_exp(d, exp, where)
+            if all and not res:
+                return False
+            if not all and res:  # match one
+                return True
+        # return if all True else False
+        return all
+
+    def __is_req_match_ctl(self, req: RequestWrapper, ctrl: ControllerFunction) -> bool:
+        _logger.debug(f"{ctrl.headers} - {ctrl.params}")
+        return self.__match_exps(req.headers, ctrl.headers, ctrl.match_all_headers_expressions, 'headers') \
+            and self.__match_exps(req.parameters, ctrl.params, ctrl.match_all_params_expressions, 'params')
+
+    def __get_ctrl(self, req: RequestWrapper) -> Tuple[ControllerFunction, Dict, List]:
+        mth = self.method.upper()
+        path = req._path
+        ctrls = self.routing_conf.get_url_controllers(path, mth)
+        for ctrl, pvs, regs in ctrls:
+            if self.__is_req_match_ctl(req, ctrl):
+                return ctrl, pvs, regs
+        return None, {}, []
+
     async def handle_request(self):
         mth = self.method.upper()
 
         req = await self.__prepare_request(mth)
-        path = req._path
 
-        ctrl, req.path_values, req.reg_groups = self.routing_conf.get_url_controller(path, mth)
+        ctrl, req.path_values, req.reg_groups = self.__get_ctrl(req)
 
         res = ResponseWrapper(self)
         if ctrl is None:
@@ -715,7 +801,7 @@ class HTTPRequestHandler:
         except HttpError as e:
             self.send_error(e.code, e.message, e.explain)
 
-    def _send_res(self, status_code: int, headers: Dict[str, str] = {}, content_type: str = "", cks: Cookies = Cookies(), body: Union[str, bytes, StaticFile] = None):
+    def __send_res_headers(self, status_code: int, headers: Dict[str, str] = {}, content_type: str = "", cks: Cookies = Cookies()):
         if "Content-Type" not in headers.keys() and "content-type" not in headers.keys():
             headers["Content-Type"] = content_type
 
@@ -733,6 +819,16 @@ class HTTPRequestHandler:
             ck = cks[k]
             self.send_header("Set-Cookie", ck.OutputString())
 
+    def _send_res_headers(self, *args, **kwargs):
+        """
+        " For response object to send and writer headers.
+        """
+        self.__send_res_headers(*args, **kwargs)
+        self.end_headers()
+
+    def _send_res(self, status_code: int, headers: Dict[str, str] = {}, content_type: str = "", cks: Cookies = Cookies(), body: Union[str, bytes, bytearray, StaticFile] = None):
+        self.__send_res_headers(status_code, headers, content_type, cks)
+
         if body is None:
             self.send_header("Content-Length", 0)
             self.end_headers()
@@ -742,7 +838,7 @@ class HTTPRequestHandler:
             self.end_headers()
             self.writer.write(data)
 
-        elif isinstance(body, bytes):
+        elif isinstance(body, bytes) or isinstance(body, bytearray):
             self.send_header("Content-Length", len(body))
             self.end_headers()
             self.writer.write(body)
